import sys

def _getprop(func):
    name = func.__name__          #<- 関数名
    vars = sys._getframe(2).f_locals #<- 呼び出し元（クラス定義）のローカル変数の一覧（辞書）
    prop = vars.get(name, None)   #<- 定義済みのプロパティ
    return prop                   #<- 定義済みのプロパティを返す

def getter(func):
    prop = _getprop(func)         #<- 定義済みのプロパティを取得
    if prop is None:              #<- 定義済みのプロパティがなければ
        return property(func)     #<-     「@property」と実質同じ
    return prop.getter(func)      #<- 「@prop.getter」と実質同じ

def setter(func):
    prop = _getprop(func)         #<- 定義済みのプロパティを取得
    if prop is None:              #<- 定義済みのプロパティがなければ
        return property(None, func)  #<- setterだけを持つプロパティを生成
    return prop.setter(func)      #<- 「@prop.setter」と実質同じ

def deleter(func):
    prop = _getprop(func)         #<- 定義済みのプロパティを取得
    if prop is None:              #<- 定義済みのプロパティがなければ
        return property(None, None, func)  #<- deleterだけを持つプロパティを生成
    return prop.setter(func)      #<- 「@prop.deleter」と実質同じ

## 使用例

class Movie(object):

    def __init__(self, title):
        self._title = title

    @setter
    def title(self, newtitle):
        self._title = newtitle

    @getter
    def title(self):
        return self._title

katasumi = Movie(u"この世界の片隅に")
print(katasumi.title)    #=> この世界の片隅に
katasumi.title = u"この世界の（さらにいくつもの）片隅に"
print(katasumi.title)    #=> この世界の（さらにいくつもの）片隅に
