class Node:   #<- 構文木のノード

    def __add__(self, other):  #<- 「+」演算子を上書き
        return TermNode(self, '+', other)

    def __mul__(self, other):  #<- 「+」演算子を上書き
        return TermNode(self, '*', other)

    def __radd__(self, other):  #<- 左右を入れ替えた「+」
        return TermNode(other, '+', self)

    def __rmul__(self, other):  #<- 左右を入れ替えた「*」
        return TermNode(other, '*', self)

class VarNode(Node):  #<- 変数(Variable)を表すノード

    def __init__(self, name):
        self.name = name    #<- 変数名

class TermNode(Node): #<- 項(Term)を表すノード

    def __init__(self, left, op, right):
        self.left  = left   #<- 演算子の左辺
        self.op    = op     #<- 演算子
        self.right = right  #<- 演算子の右辺

def inspect_node(node):  #<- 構文木を再帰的に表示
    buf = []
    def _inspect(node, depth):
        indent = "  " * depth
        if isinstance(node, TermNode):
            _inspect(node.right, depth + 1)
            buf.append(f"{indent}{node.op}\n")
            _inspect(node.left, depth + 1)
        elif isinstance(node, VarNode):
            buf.append(f"{indent}{node.name}\n")
        else:
            buf.append(f"{indent}{repr(node)}\n")
    #
    _inspect(node, 0)
    return "".join(buf)

## 使い方
a = VarNode("a")    #<- 変数を表すノード
b = VarNode("b")    #<- 変数を表すノード

print("## a * b + 2")
tree1 = a * b + 2   #<- 式を実行すると構文木ができる！
print(inspect_node(tree1))

print("## a + b * 2")
tree2 = a + b * 2   #<- 演算子の優先順位も反映される！
print(inspect_node(tree2))

print("## (a + 1) * (b + 2)")
tree3 = (a + 1) * (b + 2)   #<- もちろん丸カッコもOK！
print(inspect_node(tree3))
