import sys

def assert_(actual):                 #<- ヘルパー関数は
    return AssertionObject(actual)   #<- ヘルパーオブジェクトを返す


class AssertionObject(object):       #<- ヘルパークラス

    def __init__(self, actual):      #<- 実際の値を指定
        self._actual = actual
        self._tested = False  #<- テストされたらTrueになる

    def __del__(self):        #<- オブジェクト削除時に実行される
        if not self._tested:
            msg = "assert_(%r) is called but nothing tested." % (self._actual)
            sys.stderr.write(f"*** warning: {msg}\n")

    def __eq__(self, expected):      #<- 「==」の特殊メソッド
        self._tested = True
        actual = self._actual
        assert actual == expected, \
               "%r == %r" % (actual, expected)

    def __lt__(self, expected):      #<- 「<」の特殊メソッド
        self._tested = True
        actual = self._actual
        assert actual < expected, \
               "%r < %r" % (actual, expected)

    def is_(self, expected):      #<- 「is」用の（普通の）メソッド
        self._tested = True
        actual = self._actual
        assert actual is expected, \
               "%r is %r" % (actual, expected)

    def is_not(self, expected):   #<- 「is not」用の（普通の）メソッド
        self._tested = True
        actual = self._actual
        assert actual is not expected, \
               "%r is not %r" % (actual, expected)

    def len(self, expected):
        self._tested = True
        actual = self._actual
        assert len(actual) == expected, \
               "%r == %r" % (len(actual), expected)

    def between(self, bottom, top):
        self._tested = True
        actual = self._actual
        assert bottom <= actual <= top, \
               "%r <= %r <= %r" % (bottom, actual, top)

## 使い方
def add(x, y):          #<- テスト対象
    #return x + y       #<- 正しいコードはこうだけど、
    return x * y        #<- ここではわざと間違ってみる

def test1():            #<- テスト関数
    actual = add(1, 1)  #<- 実際の値
    expected = 2        #<- 期待する値
    #}assert_(actual) == expected
    assert_(actual).is_(expected)

#test1()                #<- テスト用関数を実行

def test2():
    assert_(['A', 'B']).len(2)
    #assert_(3.14159).between(3.1415, 3.1416)
    assert_(3.14159).between(3.1415, 3.14156)

#test2()                #<- テスト用関数を実行

def test3():
    assert_(10) is not 11   #<- assert_()は呼ばれたけど
                            #<- テスト用メソッドが呼ばれてない

test3()
