class Runner:                 #<- ランナーを表すクラス

    def __init__(self, name):
        self.name     = name
        self.distance = 0     #<- 前との距離
        self._race    = None  #<- 現在参加中のレース

    def __repr__(self):
        return f"Runner({self.name}, distance={self.distance})"

    def __neg__(self):        #<- 単項演算子「-」を上書き
        self.distance += 2    #<- 前との距離を2m増やす
        return self           #<- 重要：自分自身を返す

    def __invert__(self):     #<- 単項演算子「~」を上書き
        self.distance += 10   #<- 前との距離を10m増やす
        return self           #<- 重要：自分自身を返す

    def __lt__(self, runner): #<- 二項演算子「<」を上書き
        race = self._race     #<- 現在参加中のレース
        if race is None:      #<- 現在参加中のレースがなければ
            race = Race()     #<-   新しいレースを作成して
            self.join(race)   #<-   それに参加する
        runner.join(race)     #<- 後続のランナーもレースに参加させる
        return race           #<- 重要：レースを返す

    def join(self, race):     #<- レースに参加する
        assert self._race is None
        self._race = race     #<- インスタンス変数に設定するだけで
        race.accept(self)     #<- なく、レースのメソッドも呼び出す

    def leave(self):
        race = self._race
        self._race = None     #<- ランナーからレースへの参照をなくす
        return race

class Race:

    def __init__(self):
        self.runners = []     #<- レースに参加中のランナーのリスト

    def __bool__(self):       #<- なくてもよいが、明示してみた
        return True

    def accept(self, runner): #<- ランナーがレースに参加する
        self.runners.append(runner)

    def done(self):
        for runner in self.runners:
            runner.leave()    #<- ランナーからレースへの参照をなくす
        self.runners[:] = []  #<- レースからランナーへの参照をなくす

## 5名のランナー
A = Runner('A')
B = Runner('B')
C = Runner('C')
D = Runner('D')
E = Runner('E')
## ランナーの順番と距離を記述すると、レースが作成される
race = A < B <-- C <----- D <~~ E
## レースに参加中のランナーを一覧表示する
for runner in race.runners:
    print(runner)
race.done()
