import requests

class RequestBuilder:

    def __init__(self, meth, url):
        self._meth    = meth
        self._url     = url
        self._params  = None
        self._headers = None

    def params(self, params):
        self._params = params
        return self    #<- selfを返す（メソッドチェーンのため）

    def headers(self, headers):
        self._headers = headers
        return self    #<- selfを返す（メソッドチェーンのため）

    def send(self):
        return requests.request(self._meth, self._url,
                                params=self._params,
                                headers=self._headers)

def http(meth, url):
    return RequestBuilder(meth, url)

## 使い方
url = "https://httpbin.org/json"
payload = {"key1": "value1", "key2": "value2"}
headers = {"Content-Type": "application/json"}
response = http('GET', url).params(payload).headers(headers).send()
print(response)
