class Point:
    """経路のポイント"""

    def __init__(self, name):
        self.name = name

    _prev_route = None   #<- 直前に使った経路オブジェクト

    def __gt__(self, other):   #<- 「{{,>,}}」演算子を上書き
        route = Point._prev_route
        if route is None or route.end_point is not self:
            route = Route().append(self)
            Point._prev_route = route
        route.append(other)
        return route

    def __pos__(self):         #<- 単項演算子「{{,+,}}」を上書き
        route = Route().append(self)
        Point._prev_route = route
        return route

class Route:
    """経路"""

    def __init__(self):
        self._points = []    #<- 順番に並んだ経路ポイント

    def __str__(self):
        return " > ".join( p.name for p in self._points )

    def append(self, point):
        self._points.append(point)
        return self

    @property
    def start_point(self):
        return self._points[0] if self._points else None

    @property
    def end_point(self):
        return self._points[-1] if self._points else None

    def __gt__(self, point):   #<- 「>」演算子を上書き
        return self.append(point)

## 使い方
新宿 = Point("新宿"); 代々木 = Point("代々木")
原宿 = Point("原宿"); 渋谷   = Point("渋谷")
表参道 = Point("表参道"); 青山一丁目 = Point("青山一丁目")
route1 = + 新宿 > 代々木 > 原宿 > 渋谷     #<- JR山手線
route2 = + 渋谷 > 表参道 > 青山一丁目      #<- 地下鉄銀座線
print(route1)
print(route2)
