class Column:         #<- テーブルのカラムを表すクラス

    def __init__(self, name):
        self.name = name       #<- カラム名

    def __eq__(self, value):   #<- 「==」演算子を上書き
        op = "="               #<- SQLでは「==」ではなく「=」
        if value is None:      #<- 引数がNoneなら
            op = "is"          #<- 「is null」になるよう調整
        return f"{self.name} {op} {sql_literal(value)}"

    def __ne__(self, value):   #<- 「!=」演算子を上書き
        op = "<>"              #<- SQLでは「!=」ではなく「<>」
        if value is None:      #<- 引数がNoneなら
            op = "is not"      #<- 「is not null」になるよう調整
        return f"{self.name} {op} {sql_literal(value)}"

def sql_literal(val):  #<- Pythonの文字列や数値などをSQLに変換
    if val is None:  return "null"
    if val is True:  return "true"
    if val is False: return "false"
    if isinstance(val, int): return str(val)
    if isinstance(val, str): return _quote(val)
    if isinstance(val, Column): return val.name
    raise TypeError("%r: unknown data type." % (val,))

def _quote(s):
    s = s.replace("'", "''")         #<- 「'」をエスケープしてから
    return f"'{s}'"                  #<- 「'」で囲う

## 使い方：Pythonの式をSQLに変換する
id = Column('id')
sql = (id == 100)
print(sql)    #=> id = 100
sql = (id != 100)
print(sql)    #=> id <> 100
sql = (id == None)
print(sql)    #=> id is null
sql = (id != None)
print(sql)    #=> id is not null
