import re

## クラス変数としてテーブル名を自動付与するクラスデコレータ
def table(arg):
    ## 「@table」として使われた場合
    if isinstance(arg, type):
        cls = arg
        cls.table_name = classname2tablename(cls.__name__)
        return cls
    ## 「@table('foo_bar')」として使われた場合
    if isinstance(arg, str):
        def deco(cls):
            cls.table_name = arg
            return cls
        return deco
    ## それ以外ならTypeError
    raise TypeError("@table(%r): unexpected table name" % (name,))

## クラス名からテーブル名を生成する（単数系・複数形は考えない）
## 例：'SalesOrder' -> 'sales_order'
def classname2tablename(name):
    rexp = re.compile(r'([A-Z][^A-Z]*)')
    items = ( m.group(1).lower() for m in rexp.finditer(name) )
    return "_".join(items)

## 使用例
@table("sales_orders")
class SalesOrder:
    ...

print(SalesOrder.table_name)   #=> sales_orders
