from types import FunctionType   #<- 関数の型

## 関数デコレータ用の高階関数
def debug(arg):
    default_name = "world"
    def deco(func):        #<- 関数を返す関数
        def newfunc(name=default_name):
            ret = func(name)
            return ret
        return newfunc
    if isinstance(arg, FunctionType):  #<- 「@debug」なら
        newfunc = deco(arg)
        return newfunc     #<- 関数を返す
    else:                  #<- 「@debug(...)」なら
        default_name = arg #<- 引数の値をデフォルト値にする
        return deco        #<- 「関数を返す関数」を返す

## 関数デコレータを使用する
@debug("WORLD")            #<- nameのデフォルト値を指定する
def hello1(name):          #<- 引数「name」は省略可能になる
    print(f"Hello, {name}!")

hello1("Python")           #=> Hello Python!
hello1()                   #=> Hello WORLD!

@debug                     #<- nameのデフォルト値は"world"になる
def hello2(name):          #<- 引数「name」は省略可能になる
    print(f"Hello, {name}!")

hello2("Python")           #=> Hello Python!
hello2()                   #=> Hello world!
