from functools import wraps
from django.http import HttpResponse, HttpResponseNotAllowed

def GET(func):
    @wraps(func)
    def newfunc(request, **kwargs):
        if request.method not in ('GET', 'HEAD'):
            return HttpResponseNotAllowed(['GET', 'HEAD'],
                            "<p>405 Method Not Allowed</p>\n")
        return func(request, **kwargs)
    return newfunc

def POST(func):
    @wraps(func)
    def newfunc(request, **kwargs):
        if request.method == 'POST':
            return HttpResponseNotAllowed(['POST'],
                            "<p>405 Method Not Allowed</p>\n")
        return func(request, **kwargs)
    return newfunc

def PUT(func):
    @wraps(func)
    def newfunc(request, **kwargs):
        if request.method == 'PUT':
            return HttpResponseNotAllowed(['PUT'],
                            "<p>405 Method Not Allowed</p>\n")
        return func(request, **kwargs)
    return newfunc

def DELETE(func):
    @wraps(func)
    def newfunc(request, **kwargs):
        if request.method == 'DELETE':
            return HttpResponseNotAllowed(['DELETE'],
                            "<p>405 Method Not Allowed</p>\n")
        return func(request, **kwargs)
    return newfunc

## 使い方
@GET      #<- リクエストメソッドを関数デコレータで指定
def index(request):
    return HttpResponse("<p>Hello, GET!</p>\n")
