from django.http import HttpResponse, HttpResponseNotAllowed

## (a) GETメソッド時の処理とPOSTメソッド時の処理を、別々の関数にする。
def index_page(request):
    return HttpResponse("Hello, GET!\n")

def index_post(request):
    return HttpResponse("Hello, POST!\n")

## (c) 振り分けには、キーがリクエストメソッド、値が関数であるような
##     辞書オブジェクトを使う。
_mapping = {
    "GET": index_page, "HEAD": index_page,
    "POST": index_post,
}

## (b) リクエストメソッドに応じて処理を振り分ける関数を用意する。
def index(request, _mapping=_mapping, **kwargs):
    fn = _mapping.get(request.method, None)
    ## 対応するリクエストハンドラがなければ405 Method Not Allowed
    if fn is None:
        methods = list(_mapping.keys())
        return HttpResponseNotAllowed(methods,
                            "<p>405 Method Not Allowed</p>\n")
    return fn(request, **kwargs)

## index()の引数のデフォルト値として保存しているので、削除して構わない。
del _mapping
