from functools import wraps
from django.http import HttpResponse, HttpResponseNotAllowed
from contextlib import contextmanager

@contextmanager  #<- ジェネレータ関数をwith文で使えるようにする
def endpoint(name):
    _mapping = {}
    def on(method):
        def deco(func):
            _mapping[method] = func
            if method == 'GET':
                _mapping['HEAD'] = func
            return func
        return deco
    #
    yield on     #<- with文のブロックを実行する
    #
    def newfn(request, _mapping=_mapping, **kwargs):
        fn = _mapping.get(request.method, None)
        if fn is None:
            methods = list(_mapping.keys())
            return HttpResponseNotAllowed(methods,
                            "<p>405 Method Not Allowed</p>\n")
        return fn(request, **kwargs)
    newfn.__name__ = name
    globals()[name] = newfn  #<- グローバル変数に追加する

## 使い方
with endpoint("index") as on:

    @on("GET")
    def index_page(request):
        return HttpResponse("Hello, GET!\n")

    @on("POST")
    def index_post(request):
        return HttpResponse("Hello, POST!\n")
