from time import time
from functools import wraps

def memoize(func):
    cache = {}
    @wraps(func)
    def newfunc(*args):
        if args in cache:       #<- キャッシュされていれば、
            return cache[args]  #<- 関数を実行せずに値を返す。
        result = func(*args)    #<- キャッシュされてなければ実行し、
        cache[args] = result    #<- その結果をキャッシュする。
        return result
    return newfunc

def bench(func):      #<- 詳しくは前節を参照のこと
    t1 = time()
    func()
    t2 = time()
    label = func.__doc__ or func.__name__
    print("%s: %.3f sec" % (label, t2 - t1))

## 使い方
@memoize
def fib1(n):           #<- メモ化あり
    if n <= 1:
        return n
    else:
        return fib1(n-1) + fib1(n-2)

@bench
def _():
    """メモ化あり"""
    fib1(33)

def fib2(n):           #<- メモ化なし
    if n <= 1:
        return n
    else:
        return fib2(n-1) + fib2(n-2)

@bench
def _():
    """メモ化なし"""
    fib2(33)
