import sys

task_funcs = {}    #<- キー：タスク名、値：タスク関数

def task(func):    #<- 関数デコレータ用の高階関数
    task_name = func.__name__
    task_funcs[task_name] = func   #<- タスク関数を登録
    return func    #<- もとの関数を返す（新しい関数を作らない）

def main(args=None):
    if args is None:
        args = sys.argv[1:]  #<- argv[0]はファイル名なので除外
    task_names = args
    for task_name in task_names:
        try:
            task_func = task_funcs[task_name]  #<- タスク関数を検索
        except KeyError:    #<- タスク関数がなければエラー
            msg = f"task '{task_name}' not defined."
            sys.stderr.write(f"error: {msg}\n")
            sys.exit(1)
        task_func()         #<- タスク関数を実行
    sys.exit(0)

def run(command):           #<- UNIXコマンドを実行するヘルパー関数
    print(command)          #<- （のつもり、今は表示するだけ）
    #print(f"$ {command}")  #<- （またはプロンプトをつける）

## 使い方
@task
def compile():
    run("uplatex foo.tex")

@task
def makepdf():
    run("dvipdfmx foo.dvi")

@task
def all():
    compile()
    makepdf()

main()
