import sys
from types import FunctionType

task_funcs = {}    #<- キー：タスク名、値：タスク関数

def task(name=None):
    def deco(func, name=name):  #<- 新しい高階関数を生成
        task_name = name or func.__name__
        task_funcs[task_name] = func   #<- タスク関数を登録
        return func
    if isinstance(name, FunctionType):  #<- 引数が関数なら
        func = name
        return deco(func, None)  #<- 関数を返す
    else:                   #<- 引数が関数でなければ
        return deco         #<- 生成した高階関数を返す

def main(args=None):
    if args is None:
        args = sys.argv[1:]  #<- argv[0]はファイル名なので除外
    task_names = args
    for task_name in task_names:
        try:
            task_func = task_funcs[task_name]
        except KeyError:    #<- タスク関数がなければエラー
            msg = f"task '{task_name}' not defined."
            sys.stderr.write(f"error: {msg}\n")
            sys.exit(1)
        task_func()         #<- タスク関数を実行
    sys.exit(0)

def run(command):           #<- UNIXコマンドを実行するヘルパー関数
    print(command)          #<- （のつもり、今は表示するだけ）
    #print(f"$ {command}")  #<- （またはプロンプトをつける）

## 使い方
@task
def compile():
    run("uplatex foo.tex")

@task
def makepdf():
    run("dvipdfmx foo.dvi")

@task("all")                #<- タスク名を指定すれば
def task_all():             #<- タスク名とは違う関数名が使える
    compile()
    makepdf()

main()
