import inspect

## 関数が「*args」を使っているか
def a1(*args): pass     #<- 使っている
def a2(args):  pass     #<- 使っていない
print(a1.__code__.co_flags & inspect.CO_VARARGS)  #=> 4
print(a2.__code__.co_flags & inspect.CO_VARARGS)  #=> 0

## 関数が「**kwargs」を使っているか
def b1(**kwargs): pass  #<- 使っている
def b2(kwargs):   pass  #<- 使っていない
print(b1.__code__.co_flags & inspect.CO_VARKEYWORDS)  #=> 8
print(b2.__code__.co_flags & inspect.CO_VARKEYWORDS)  #=> 0

## 関数が入れ子になっているか
def tmp1():
    def c1(): pass    #<- 入れ子になっている
    print(c1.__code__.co_flags & inspect.CO_NESTED) #=> 16
tmp1()
def c2(): pass        #<- 入れ子になっていない
print(c2.__code__.co_flags & inspect.CO_NESTED)     #=> 0

## ジェネレータ関数かどうか
def d1(): yield 1     #<- ジェネレータ関数である
def d2(): pass        #<- ジェネレータ関数でない
print(d1.__code__.co_flags & inspect.CO_GENERATOR) #=> 32
print(d2.__code__.co_flags & inspect.CO_GENERATOR) #=> 0

## 入れ子の関数が外側のローカル変数を参照していないか
def tmp2():
    x = 10                #<- 外側のローカル変数を
    def e1(): return 10   #<- 参照していない
    def e2(): return x    #<- 参照している
    print(e1.__code__.co_flags & inspect.CO_NOFREE) #=> 64
    print(e2.__code__.co_flags & inspect.CO_NOFREE) #=> 0
tmp2()

## コルーチンかどうか
async def f1(): pass      #<- コルーチンである
def f2(): pass            #<- コルーチンでない
print(f1.__code__.co_flags & inspect.CO_COROUTINE) #=> 128
print(f2.__code__.co_flags & inspect.CO_COROUTINE) #=> 0

## Generatorベースのコルーチンかどうか
import asyncio
@asyncio.coroutine
def g1(): yield from []   #<- Yes (@asyncio.coroutineがあるから)
def g2(): yield from []   #<- No
import inspect
print(g1.__code__.co_flags & inspect.CO_ITERABLE_COROUTINE) #=> 256
print(g2.__code__.co_flags & inspect.CO_ITERABLE_COROUTINE) #=> 0

## 非同期ジェネレータかどうか
async def h1(): yield 1   #<- Yes
async def h2(): await 1   #<- No
print(h1.__code__.co_flags & inspect.CO_ASYNC_GENERATOR)  #=> 512
print(h2.__code__.co_flags & inspect.CO_ASYNC_GENERATOR)  #=> 0
