import sys
import ast

def ast_traverse(node, depth=0):
    """抽象構文木を深さつきでたどる"""
    yield node, depth
    for child in ast.iter_child_nodes(node):
        yield from ast_traverse(child, depth+1)

def ast_dump(node, depth=0, *, color=None):
    """抽象構文木をインデントつきで表示する"""
    if color is None:
        color = _isatty(sys.stdout)  #<- 標準出力が端末ならカラー化
    if color:
        format = lambda i: "\033[0;33m<#%02d>\033[0m" % i
    else:
        format = lambda i: "<#%02d>" % i
    #
    i = 0
    d = {}   # {id: i}
    def _repr(v):
        nonlocal i
        if isinstance(v, ast.AST):
            if id(v) in d:
                return d[id(v)]
            i += 1
            d[id(v)] = idx = format(i)
            return idx
            # or: return d.setdefault(id(v), format(i))
        if isinstance(v, list):
            return "[" + ", ".join( _repr(x) for x in v ) + "]"
        return repr(v)
    #
    buf = []
    for x, depth in ast_traverse(node, depth=depth):
        indent = "  " * depth
        classname = x.__class__.__name__
        if color:
            classname = f"\033[1;34m{classname}\033[0m"
        idx = d.get(id(x), None)
        if idx is None:
            idx = d[id(x)] = _repr(x)
        fields = ", ".join( f"{k}={_repr(v)}"
                               for k, v in ast.iter_fields(x) )
        line = f"{indent}{idx} {classname}({fields})"
        buf.append(line)
    return "\n".join(buf)

def _isatty(stdout):
    """出力先が端末かどうか調べる（端末ならTrueを返す）"""
    if hasattr(stdout, 'buffer'):
        return stdout.buffer.isatty()
    else:
        return stdout.isatty()

## 使い方
program = "y = x + 1"
node = ast.parse(program)
print(ast_dump(node))
