from contextlib import contextmanager

@contextmanager
def mixin(obj, klass):
    """オブジェクトにクラスをミックスインし、元に戻す"""
    original = obj.__class__   #<- 元のクラス
    class MixIn(klass, original):  #<- 多重継承した一時クラス
        pass
    obj.__class__ = MixIn      #<- 所属クラスを一時的に変更
    yield                      #<- with文のブロックを実行
    obj.__class__ = original   #<- 元のクラスに戻す

class 取引先:
    def __init__(self, id, name):
        self._id   = id
        self._name = name

    id   = property(lambda self: self._id)    #<- 簡潔なプロパティ定義
    name = property(lambda self: self._name)  #<- 簡潔なプロパティ定義

class 仕入先:   #<- 取引先クラスを継承しない
    def 発注(self, 商品, 個数):
        print(f"{self.name}へ{商品}を{個数}個、発注しました。")

class 販売先:   #<- 取引先クラスを継承しない
    def 受注(self, 商品, 個数):
        print(f"{self.name}から{商品}を{個数}個、受注しました。")

## ある取引先を…
company = 取引先(100, 'カウプラン機関')

## 仕入先として使ったり、
with mixin(company, 仕入先):
    print(company.__class__)  #=> <class 'MixIn'> （変更された！）
    company.発注('合成化合物', 200)
print(company.__class__)      #=> <class '取引先'> （元に戻った！）

## 販売先として使う。
with mixin(company, 販売先):
    print(company.__class__)  #=> <class 'MixIn'>  （変更された！）
    company.受注('化学薬品', 500)
print(company.__class__)      #=> <class '取引先'> （元に戻った！）
