class CachedProperty:  #<- 自動的にキャッシュするプロパティクラス

    __slots__ = ('_func', '_name')

    def __init__(self, getter_func):
        self._func  = getter_func
        self._name  = "_" + getter_func.__name__

    def __get__(self, obj, klass):  #<- ディスクリプタ用メソッド
        name = self._name         #<- キャッシュ用のインスタンス変数名
        if hasattr(obj, name):    #<- すでにキャッシュ済みなら
            return getattr(obj, name) #<- その値を返す
        value = self._func(obj)   #<- getter用関数を実行して、それを
        setattr(obj, name, value) #<- キャッシュ用のインスタンス変数に
        return value              #<- 設定してから、値を返す

domain = "example.com"

class APIClient:

    def __init__(self, api_ver):
        self.api_ver = api_ver

    @CachedProperty
    def base_url(self):
        return f"https://{domain}/api/v{self.api_ver}"  #<- ベースURLを返す

client = APIClient(2)
print(client.base_url)   #=> https://example.com/api/v2
print(client._base_url)  #<- インスタンス変数にキャッシュされている
