class CachedProperty:

    __slots__ = ('_func', )

    def __init__(self, getter_func):
        self._func  = getter_func

    def __get__(self, obj, klass):
        name = self._func.__name__  #<- プロパティの名前を使う
        value = self._func(obj)
        setattr(obj, name, value)  #<- プロパティと同名のインスタンス変数で上書き
        return value

domain = "example.com"

class APIClient:

    def __init__(self, api_ver):
        self.api_ver = api_ver

    @CachedProperty
    def base_url(self):
        return f"https://{domain}/api/v{self.api_ver}"

client = APIClient(2)
print(client.__dict__)  #=> {'api_ver': 2}
print(client.base_url)  #<- 1回目のアクセスではgetter関数が呼び出される
print(client.__dict__)  #=> {'api_ver': 2, 'base_url': 'https://example.com/api/v2'}
print(client.base_url)  #<- 2回目以降のアクセスでは呼び出されない
