class Attr1:    #<- 非データディスクリプタ用クラス（__set__()がないので）
    def __get__(self, obj, klass):
        return obj.name

class Attr2:    #<- データディスクリプタ用クラス（__set__()があるので）
    def __get__(self, obj, klass):
        return obj.name
    def __set__(self, obj, value):
        obj.name = value

class Dummy:

    def __init__(self, name):
        self.name = name

    attr1 = Attr1()   #<- 非データディスクリプタ（読み出し専用）
    attr2 = Attr2()   #<- データディスクリプタ（読み書き両用）

obj = Dummy("world")

## 上書き前
print(obj.attr1)    #=> world
print(obj.attr2)    #=> world

## インスタンス変数で上書き
## ・非データディスクリプタはインスタンス変数で上書きできる
## ・データディスクリプタはインスタンス変数で上書きできない
obj.__dict__['attr1'] = "WORLD"  #<- 属性attr1の値も上書きされる
obj.__dict__['attr2'] = "WORLD"  #<- 属性attr2の値は上書きされない
print()

## 上書き後
print(obj.attr1)    #=> WORLD （属性の値が上書きされた）
print(obj.attr2)    #=> world （属性の値が上書きされない）

## 代入による上書き
## ・非データディスクリプタの場合はインスタンス変数への代入
## ・データディスクリプタの場合は__set__()を呼び出す
print()
obj.attr1 = "WORLD!!"     #<- インスタンス変数attr1を上書き
obj.attr2 = "WORLD!!"     #<- 属性attr2の値を上書き
print(obj.attr1)          #=> WORLD!!
print(obj.attr2)          #=> WORLD!!
