import re

## クラス名からテーブル名を生成する（簡単化のため連続した大文字は考慮せず）
## 例：'SalesOrder' -> 'sales_order'
def classname2tablename(name):
    rexp = re.compile(r'([A-Z][^A-Z]*)')
    items = ( m.group(1).lower() for m in rexp.finditer(name) )
    return "_".join(items)

## メタクラス
class TableMetaclass(type):
    def __init__(self, name, bases, dict_):
        type.__init__(self, name, bases, dict_)
        ## テーブル名を自動設定する（ただしTableクラスを除く）
        if name != 'Table':  #<- または if bases[0] is not object:
            self.table_name = classname2tablename(name)

## テーブルクラス（他のクラスの親クラス）
class Table(object, metaclass=TableMetaclass):
    pass

## 受注クラス（sales_orderテーブルに対応）
class SalesOrder(Table):
    pass

## 管理者ユーザステータスクラス（admin_user_statusテーブルに対応）
class AdminUserStatus(Table):
    pass

## テーブル名が自動設定されたことを確認
print(SalesOrder.table_name)        #=> sales_order
print(AdminUserStatus.table_name)   #=> admin_user_status
