import re

## クラス名からテーブル名を生成する（簡単化のため連続した大文字は考慮せず）
## 例：'SalesOrder' -> 'sales_order'
def classname2tablename(name):
    rexp = re.compile(r'([A-Z][^A-Z]*)')
    items = ( m.group(1).lower() for m in rexp.finditer(name) )
    return "_".join(items)

## メタクラス
class TableMetaclass(type):
    def __init__(self, name, bases, dict_):
        type.__init__(self, name, bases, dict_)
        ## テーブル名を自動設定する（ただしTableクラスを除く）
        if name != 'Table':  #<- または if bases[0] is not object:
            self.table_name = classname2tablename(name)
        ## Columnオブジェクトの名前を自動設定する
        for k in dir(self):
            v = getattr(self, k)
            if isinstance(v, Column):
                if v.name == None:
                    v.name = k

## テーブルのカラム情報
class Column(object):
    def __init__(self, name, type=None, *, pk=False, notnull=True):
        if not isinstance(name, str):  #<- 「Column(int)」なら
            name, type = None, name    #<- 「Column(None, int)」とみなす
        if type is None:
            raise TypeError("Column(%r): type required." % (name,))
        self.name    = name     #<- カラム名
        self.type    = type     #<- データ型
        self.pk      = pk       #<- 主キー(primary key)制約
        self.notnull = notnull  #<- not null制約
        if pk:
            self.notnull = True #<- 主キーなら自動的にnot nullにする

## テーブルクラス（他のクラスの親クラス）
class Table(object, metaclass=TableMetaclass):
    pass

## 受注クラス（sales_orderテーブルに対応）
class SalesOrder(Table):
    id        = Column( int, pk=True)
    item      = Column( str)
    count     = Column( int)
    unitprice = Column( int)

## 管理者ユーザステータスクラス（admin_user_statusテーブルに対応）
class AdminUserStatus(Table):
    code      = Column( str, pk=True)
    name      = Column( str)
    desc      = Column( str, notnull=False)

## カラム名が自動設定されたことを確認
print(SalesOrder.unitprice.name)   #=> unitprice
print(AdminUserStatus.code.name)   #=> code
