import os
from random import randint

class GeneratorContextManager:

    def __init__(self, generator):     #<- ジェネレータを受け取る
        self.generator = generator

    def __enter__(self):
        value = next(self.generator)   #<- yield文までを実行
        return value                   #<- yield文の引数をそのまま返す

    def __exit__(self, exc_type, exc_value, traceback):
        if exc_value is None:  #<- with文のブロックで例外が発生しなければ
            try:
                next(self.generator)   #<- yield文の続きを実行
            except StopIteration:
                pass

def contextmanager(func):     #<- 関数デコレータ用
    def newfunc(*args, **kwds):
        generator = func(*args, **kwds)
        return GeneratorContextManager(generator)
    return newfunc

## 使用例
@contextmanager
def f1(value):
    print("** enter")   #<- 前処理
    yield value         #<- with文のブロックが実行される
    print("** exit")    #<- 後処理

with f1(123) as x:
    print(f"x={x}")     #<- with文のブロック（例外が発生しない）
