from time import time
from contextlib import contextmanager

@contextmanager
def bench(title):
    start = time()
    yield
    elapsed = time() - start
    print("%-20s %6.3f sec" % (title, elapsed))

## 使用例
strings = ("A"*10, "B"*10, "C"*10, "D"*10, "E"*10)
N = 1000000

with bench("'+' operator"):
    s1, s2, s3, s4, s5 = strings
    for _ in range(N):
        result = s1 + s2 + s3 + s4 + s5

with bench("'%' operator"):
    s1, s2, s3, s4, s5 = strings
    for _ in range(N):
        result = "%s%s%s%s%s" % (s1, s2, s3, s4, s5)

with bench('"".join(tuple)'):
    s1, s2, s3, s4, s5 = strings
    for _ in range(N):
        result = "".join((s1, s2, s3, s4, s5))
