from time import time
from contextlib import contextmanager

class Bench:

    def __init__(self):
        self.results = []

    @contextmanager
    def __call__(self, title):  #<- 関数のように呼び出すための特殊メソッド
        start = time()
        yield
        elapsed = time() - start
        print("%-20s %6.3f sec" % (title, elapsed))
        self.results.append((elapsed, title))

    def show_ranking(self, reverse=False):
        tuples = sorted(self.results, reverse=reverse)
        for i, t in enumerate(tuples, 1):
            elapsed, title = t
            print("%2d. %-20s %6.3f sec" % (i, title, elapsed))

    def __enter__(self):
        return self

    def __exit__(self, *args):
        if args[1] is None:
            print("----------------------------------------")
            self.show_ranking()

## 使用例
strings = ("A"*10, "B"*10, "C"*10, "D"*10, "E"*10)
N = 1000000

with Bench() as bench:

    with bench("'+' operator"):
        s1, s2, s3, s4, s5 = strings
        for _ in range(N):
            result = s1 + s2 + s3 + s4 + s5

    with bench("'%' operator"):
        s1, s2, s3, s4, s5 = strings
        for _ in range(N):
            result = "%s%s%s%s%s" % (s1, s2, s3, s4, s5)

    with bench('"".join(tuple)'):
        s1, s2, s3, s4, s5 = strings
        for _ in range(N):
            result = "".join((s1, s2, s3, s4, s5))

