from time import time
from contextlib import contextmanager

class Bench:

    def __init__(self):
        self.results = []

    def __call__(self, title):  #<- 関数のように呼び出すための特殊メソッド
        def deco(func):
            start = time()
            func()
            elapsed = time() - start
            print("%-20s %6.3f sec" % (title, elapsed))
            self.results.append((elapsed, title))
        return deco

    def show_ranking(self, reverse=False):
        tuples = sorted(self.results, reverse=reverse)
        for i, t in enumerate(tuples, 1):
            elapsed, title = t
            print("%2d. %-20s %6.3f sec" % (i, title, elapsed))

    def __enter__(self):
        return self

    def __exit__(self, *args):
        if args[1] is None:
            print("----------------------------------------")
            self.show_ranking()

## 使用例
strings = ("A"*10, "B"*10, "C"*10, "D"*10, "E"*10)
N = 1000000

with Bench() as bench:

    @bench("'+' operator")
    def _():
        s1, s2, s3, s4, s5 = strings
        for _ in range(N):
            result = s1 + s2 + s3 + s4 + s5

    @bench("'%' operator")
    def _():
        s1, s2, s3, s4, s5 = strings
        for _ in range(N):
            result = "%s%s%s%s%s" % (s1, s2, s3, s4, s5)

    @bench('"".join(tuple)')
    def _():
        s1, s2, s3, s4, s5 = strings
        for _ in range(N):
            result = "".join((s1, s2, s3, s4, s5))
