import sys

## 自作の例外ハンドラ
def errorhandler(exc_type, exc, exc_traceback):
    ## except節で別の例外が発生した場合
    if exc.__context__:
        _errorhandler(exc.__context__)
        sys.stderr.write("""
During handling of the above exception, another exception occurred:
\n""")
    ## raise ... from exceptionのように指定した場合
    if exc.__cause__:
        _errorhandler(exc.__context__)
        sys.stderr.write("""
The above exception was the direct cause of the following exception:
\n""")
    ##
    _errorhandler(exc)

def _errorhandler(exc):
    sys.stderr.write("\n".join(__errorhandler(exc)))
    sys.stderr.write("\n")

def __errorhandler(exc):
    yield "Traceback (most recent call last):"
    for t in each_traceback(exc):
        filename, lineno, funcname, linetext = t
        yield f'  File "{filename}", line {lineno} in {funcname}'
        if linetext:
            yield f'    {linetext.strip()}'
    yield f"{exc.__class__.__name__}: {exc}"

## トレースバックごとにファイル名と行番号と関数名と行文字列をyield
def each_traceback(exc):
    cache = {}     #<- {filename: [line]}
    tb = exc.__traceback__
    while tb is not None:
        code = tb.tb_frame.f_code
        filename = code.co_filename  #<- ファイル名
        lineno   = tb.tb_lineno      #<- 行番号
        funcname = code.co_name      #<- 関数名
        linetext = _line_content(filename, lineno, cache) #<- 行
        yield filename, lineno, funcname, linetext
        tb = tb.tb_next
    cache.clear()

## ファイル名と行番号を指定したらその行を文字列として返す
def _line_content(filename, lineno, cache, encoding='utf-8'):
    if filename not in cache:
        try:
            with open(filename, 'r', encoding=encoding) as f:
                cache[filename] = f.readlines()
        except:
            cache[filename] = None
    try:
        return cache[filename][lineno-1]
    except:
        return None

## 例外ハンドラとして設定
sys.excepthook = errorhandler

##
def f1(): f2()
def f2(): f3()
def f3(): blabla()  #=> NameError: name 'blabla' is not defined
f1()
