import sys

## 自作の例外ハンドラ
def errorhandler(exc_type, exc, exc_traceback):
    ## except節で別の例外が発生した場合
    if exc.__context__:
        _errorhandler(exc.__context__)
        sys.stderr.write("""
During handling of the above exception, another exception occurred:
\n""")
    ## raise ... from exceptionのように指定した場合
    if exc.__cause__:
        _errorhandler(exc.__context__)
        sys.stderr.write("""
The above exception was the direct cause of the following exception:
\n""")
    ##
    _errorhandler(exc)

def _errorhandler(exc):
    sys.stderr.write("\n".join(__errorhandler(exc)))
    sys.stderr.write("\n")

def __errorhandler(exc):
    c1 = mono   #<- 1行目には色をつけない
    c2 = blue   #<- ファイル名や行番号は「青」
    c3 = mono   #<- 行の内容は色をつけない
    c4 = red    #<- 例外クラスとエラーメッセージは「赤」
    if not _isatty(sys.stderr):  #<- 標準エラー出力が端末でないなら
        c1 = c2 = c3 = c4 = mono #<- 色をつけない
    yield c1("Traceback (most recent call last):")
    for t in each_traceback(exc):
        filename, lineno, funcname, linetext = t
        yield c2(f'  File "{filename}", line {lineno} in {funcname}')
        if linetext:
            yield c3(f'    {linetext.strip()}')
    yield c4(f"{exc.__class__.__name__}: {exc}")

## 文字列をANSIカラー化する
def mono(s):    return s
def black(s):   return f"\033[0;30m{s}\033[0m"
def red(s):     return f"\033[0;31m{s}\033[0m"
def green(s):   return f"\033[0;32m{s}\033[0m"
def yellow(s):  return f"\033[0;33m{s}\033[0m"
def blue(s):    return f"\033[0;34m{s}\033[0m"
def magenta(s): return f"\033[0;35m{s}\033[0m"
def syan(s):    return f"\033[0;36m{s}\033[0m"
def white(s):   return f"\033[0;37m{s}\033[0m"

def _isatty(out):
    """出力先が端末かどうか調べる（端末ならTrueを返す）"""
    if hasattr(out, 'buffer'):
        return out.buffer.isatty()
    else:
        return out.isatty()

## トレースバックごとにファイル名と行番号と関数名と行文字列をyield
def each_traceback(exc):
    cache = {}     #<- {filename: [line]}
    tb = exc.__traceback__
    while tb is not None:
        code = tb.tb_frame.f_code
        filename = code.co_filename  #<- ファイル名
        lineno   = tb.tb_lineno      #<- 行番号
        funcname = code.co_name      #<- 関数名
        linetext = _line_content(filename, lineno, cache) #<- 行
        yield filename, lineno, funcname, linetext
        tb = tb.tb_next
    cache.clear()

## ファイル名と行番号を指定したらその行を文字列として返す
def _line_content(filename, lineno, cache, encoding='utf-8'):
    if filename not in cache:
        try:
            with open(filename, 'r', encoding=encoding) as f:
                cache[filename] = f.readlines()
        except:
            cache[filename] = None
    try:
        return cache[filename][lineno-1]
    except:
        return None

## 例外ハンドラとして設定
sys.excepthook = errorhandler

##
def f1(): f2()
def f2(): f3()
def f3(): blabla()  #=> NameError: name 'blabla' is not defined
f1()
