import os
import json

class JSONSyntaxError(SyntaxError):
    pass

filename = "ex-jsonerror.json"
try:
    with open(filename) as f:
        json_data = json.load(f)
except json.decoder.JSONDecodeError as ex:
    ## jsonモジュールのトレースバックは表示しない
    pylibpath = os.path.dirname(json.__file__)
    tb = ex.__traceback__
    while tb.tb_next is not None:
        pyfile = tb.tb_next.tb_frame.f_code.co_filename
        if pyfile.startswith(pylibpath):
            tb.tb_next = tb.tb_next.tb_next
        else:
            tb = tb.tb_next
    ## JSONファイルのエラー箇所を表示する
    errmsg = ex.args[0]
    lineno = ex.lineno
    column = ex.colno
    with open(filename) as f:
        line = f.readlines()[lineno-1]
    newex = JSONSyntaxError(errmsg, (filename, lineno, column, line))
    newex.__traceback__ = ex.__traceback__
    raise newex from None
