import sys
import testcase
__skipTraceback = True

class TestRunner:         #<- テストランナー

    def run(self):
        ## TestCaseクラスの子クラスをインスタンス化して実行する
        for cls in testcase.TestCase.__subclasses__():
            for name in dir(cls):
                if name.startswith('test'):
                    self._run(cls, name)

    def _run(self, cls, name):
        result = 'ok'; ex = None
        try:
            cls().run(name)   #<- インスタンス化して実行
        except AssertionError as _ex:  #<- アサーション例外
            result = 'Fail'; ex = _ex
        except Exception as _ex:       #<- それ以外の例外
            result = 'Error'; ex = _ex
        print(f"- [{result}] {cls.__name__}.{name}()")
        if ex:   #<- 例外があればトレースバックを表示
            ## 例外オブジェクトのトレースバックを間引いて表示
            ex.__traceback__ = self._reduce_traceback(ex.__traceback__)
            sys.excepthook(type(ex), ex, ex.__traceback__)

    def _reduce_traceback(self, tb):
        ## ダミーのトレースバックを作成し、連結リストの先頭にする
        head = type(tb)(tb, tb.tb_frame, 0, 0)  #<- ダミーを作成
        head.tb_next = tb              #<- 連結リストの先頭にする
        ## 連結リストをたどる
        tb = head
        while tb.tb_next is not None:
            ## スタックフレーム経由でグローバル変数の一覧にアクセスし、
            ## '__skipTraceback' が設定されてあれば間引く（スキップする）
            globalvars = tb.tb_next.tb_frame.f_globals
            if globalvars.get('__skipTraceback'):
                tb.tb_next = tb.tb_next.tb_next
            else:
                tb = tb.tb_next
        ## ダミーを除いた連結リストを返す
        return head.tb_next

def main():
    TestRunner().run()
